package com.jz.dm.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.jz.common.constant.ResultMsg;
import com.jz.common.utils.HttpsUtils;
import com.jz.common.utils.Result;
import com.jz.dm.common.enums.GeneralStatusTypeEnum;
import com.jz.dm.common.enums.apiInterface.ApiStatusEnum;
import com.jz.dm.common.enums.auth.AuthModeEnum;
import com.jz.dm.mapper.ApiAuthMapper;
import com.jz.dm.mapper.ApiInterfaceMapper;
import com.jz.dm.models.domian.ApiAuth;
import com.jz.dm.models.domian.ApiInterface;
import com.jz.dm.models.domian.ApiInterfaceCustom;
import com.jz.dm.models.dto.ApiCallDataDto;
import com.jz.dm.models.req.api.ApiInterfaceDetailReq;
import com.jz.dm.models.req.api.ApiInterfaceInfoListReq;
import com.jz.dm.service.ApiInterfaceService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * @author ZC
 * @PACKAGE_NAME: com.jz.dm.service.impl
 * @PROJECT_NAME: jz-dm-parent
 * @NAME: ApiInterfaceServiceImpl
 * @DATE: 2021-1-2/14:27
 * @DAY_NAME_SHORT: 周六
 * @Description:
 **/
@Service("apiInterfaceService")
@Slf4j
public class ApiInterfaceServiceImpl implements ApiInterfaceService {

    /**
     * 测试url
     */
    public static final String testUrl ="";

    @Resource
    private ApiInterfaceMapper apiInterfaceMapper;
    @Resource
    private ApiAuthMapper apiAuthMapper;
    @Autowired
    private HttpsUtils httpsUtils;

    /**
     * 获取API列表信息
     *
     * @param req
     * @return
     */
    @Override
    public IPage<ApiInterface > listApiInterface(ApiInterfaceInfoListReq req) {
        IPage<ApiInterface > page = new Page<>(req.getPageNum(), req.getPageSize());
        QueryWrapper<ApiInterface > query = new QueryWrapper<>();
        /*if (StringUtils.isNotBlank(req.getApiKey())){
            query.eq("api_key",req.getApiKey());
        }*/
       /* if (StringUtils.isNotBlank(req.getStatus())){
            query.eq("status",req.getStatus());
        }*/
        query.eq("ai.is_deleted", 0);
        query.orderByDesc("ai.create_date");
        query.groupBy("ai.create_date");
        return apiInterfaceMapper.listApiInterfaceInfo(page, query);

    }

    /**
     * 获取API详情信息
     *
     * @param req
     * @return
     */
    @Override
    public Result getApiInterfaceDetail(ApiInterfaceDetailReq req) {
        return Result.of_success(apiInterfaceMapper.selectDetail(req));
    }

    /**
     * 删除API信息
     *
     * @param apiKey
     * @return
     */
    @Override
    public Result delApiInterface(String apiKey) {
        ApiInterface apiInterface = getReqTargetInfo(apiKey);
        if (null == apiInterface) {
            Result.of_error(ResultMsg.DATA_NOT_FOUND);
        }
        if (ApiStatusEnum.EXPIRY.name().equals(apiInterface.getStatus())) {
            return Result.of_error("API已失效无法再次操作!");
        }
        Integer authStat = getAuthCall(apiKey, AuthModeEnum.POWER_CALL_MODE.name(), null);
        Integer authDateStat = getAuthCall(apiKey, AuthModeEnum.PERMANENT_TIME_MODE.name(), new Date());
        UpdateWrapper<ApiInterface> updateInfo = new UpdateWrapper<>();
        if (authStat + authDateStat > 0) { //有已授权未使用的先将状态置为$$ 下架
            updateInfo.set("status", ApiStatusEnum.SOLDOUT);//下架状态%%可以调用,但不可以购买
        } else {
            updateInfo.set("status", ApiStatusEnum.EXPIRY);//失效状态
            updateInfo.set("is_deleted", 1);
        }
        updateInfo.eq("id", apiInterface.getId());
        if (apiInterfaceMapper.update(null, updateInfo) > 0) {
            return Result.of_success(ResultMsg.UPDATE_SUCCESS);
        }
        return Result.of_success(ResultMsg.UPDATE_FAIL);
    }

    /**
     * dmp 删除处理
     *
     * @param apiKey
     * @return
     */
    @Override
    public Result dmpDeleteAuth(String type, String apiKey) {
        if ("0".equals(type)) {//确认是否删除
            Integer call = getAuthCall(apiKey, AuthModeEnum.PERMANENT_TIME_MODE.name(), null);
            if (call > 0) {
                return Result.of_error("已存在授权用户,是否要删除!");
            }
            return Result.of_success(ResultMsg.SUCCESS);
        } else if ("1".equals(type)) { //正式删除
            ApiInterface apiInterface = getApiInfo(apiKey);
            if (null == apiInterface) {
                return Result.of_error(ResultMsg.DATA_NOT_EXIST);
            }
            if (ApiStatusEnum.EXPIRY.name().equals(apiInterface.getStatus())) {
                return Result.of_error("API已失效无法再次操作!");
            }
            updateApiAuthStatus(apiInterface.getId());
            UpdateWrapper<ApiInterface> updateWra = new UpdateWrapper<>();
            updateWra.set("status", ApiStatusEnum.EXPIRY);
            updateWra.set("is_deleted", 1);
            updateWra.eq("id", apiInterface.getId());
            if (apiInterfaceMapper.update(null, updateWra) == 0) {
                return Result.of_error(ResultMsg.UPDATE_FAIL);
            }
        }
        return Result.of_success(ResultMsg.UPDATE_SUCCESS);
    }

    /**
     * 更新已经认证的API
     *
     * @param apiInterfaceId
     */
    private void updateApiAuthStatus(Long apiInterfaceId) {
        QueryWrapper<ApiAuth> queryWra = new QueryWrapper<>();
        queryWra.eq("api_interface_id", apiInterfaceId);
        queryWra.eq("handler", 0);//主要是确认单次调用是否调用
        queryWra.eq("auth_mode", AuthModeEnum.PERMANENT_TIME_MODE);
        queryWra.eq("status", GeneralStatusTypeEnum.VALID);
        queryWra.eq("is_deleted", 0);
        List<ApiAuth> authList = apiAuthMapper.selectList(queryWra);
        if (!CollectionUtils.isEmpty(authList)) {
            apiAuthMapper.updateAuthBatch(authList);
        }
    }

    /**
     * 获取单次调用有效授权
     *
     * @param apiKey
     * @param modeType
     * @param date
     * @return
     */
    private Integer getAuthCall(String apiKey, String modeType, Date date) {
        //按次调用统计
        QueryWrapper<ApiAuth> query = new QueryWrapper<>();
        query.eq("api_key", apiKey);
        query.eq("handler", 0);//主要是确认单次调用是否调用
        query.eq("auth_mode", modeType);
        if (null != date) {
            query.gt("valid_end_time", date);
        }
        query.eq("status", GeneralStatusTypeEnum.VALID);
        query.eq("is_deleted", 0);
        return apiAuthMapper.selectCount(query);
    }


    /**
     * 获取有效授权总数
     *
     * @param apiInterface
     * @return
     */
    @Override
    public Integer getValidAuthStatus(ApiInterface apiInterface) {
        Integer authCall = getAuthCall(apiInterface.getApiKey(), AuthModeEnum.POWER_CALL_MODE.name(), null);
        Integer authDateCall = getAuthCall(apiInterface.getApiKey(), AuthModeEnum.PERMANENT_TIME_MODE.name(), new Date());
        return authCall + authDateCall;
    }

    /**
     * api测试接口
     *
     * @param apiKey
     * @return
     */
    @Override
    public Result apiTestInterface(String apiKey) {
       ApiInterfaceCustom apiInterfaceCustom= apiInterfaceMapper.getTestData(apiKey);
        if (null == apiInterfaceCustom) {
            return Result.of_error(ResultMsg.DATA_NOT_FOUND);
        }
       /* if (StringUtils.isNotBlank(apiInterfaceCustom.getApiExample())){
            httpsUtils.doGet(testUrl,"");
        }else {

        }*/
        return Result.of_success(ResultMsg.SUCCESS);
    }



    /**
     * 获取用户授权信息
     *
     * @param apiKey
     * @param authCode
     * @return
     */
    @Override
    public ApiAuth getApiAuthInfo(String apiKey, String authCode) {
        QueryWrapper<ApiInterface> query = new QueryWrapper<>();
        query.eq("is_deleted", 0);
        query.eq("api_key", apiKey);
        ApiInterface apiInterface = apiInterfaceMapper.selectOne(query);
        if (null != apiInterface) {
            QueryWrapper<ApiAuth> queryWrapper = new QueryWrapper<>();
            queryWrapper.eq("is_deleted", 0);
            queryWrapper.eq("auth_code", authCode);
            ApiAuth apiAuth = apiAuthMapper.selectOne(queryWrapper);
            if (null != apiAuth) {
                return apiAuth;
            }
        }
        return null;
    }

    /**
     * 获取API信息
     *
     * @param apiKey
     * @return
     */
    @Override
    public ApiInterface getApiInfo(String apiKey) {
        QueryWrapper<ApiInterface> query = new QueryWrapper<>();
        query.eq("is_deleted", 0);
        query.eq("api_key", apiKey);
        return apiInterfaceMapper.selectOne(query);
    }

    @Override
    public ApiInterface getReqTargetInfo(String apiKey) {
        QueryWrapper<ApiInterface> query = new QueryWrapper<>();
        query.eq("api_key", apiKey);
        query.eq("status", ApiStatusEnum.ISSUE.name());
        query.eq("is_deleted", 0);
        return apiInterfaceMapper.selectOne(query);
    }
}
