package com.jz.dm.mall.moduls.service.impl;


import com.baomidou.mybatisplus.extension.api.R;
import com.jz.common.constant.ResultMsg;
import com.jz.common.utils.DateUtils;
import com.jz.common.utils.Result;
import com.jz.common.utils.StatusCode;
import com.jz.dm.mall.moduls.controller.finance.bean.FinanceCustomerAssetsReq;
import com.jz.dm.mall.moduls.entity.FinanceCustomerAssets;
import com.jz.dm.mall.moduls.mapper.FinanceCustomerAssetsDao;
import com.jz.dm.mall.moduls.service.FinanceCustomerAssetsService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import java.math.BigDecimal;
import java.util.Date;

/**
 * 商城企业客户资产(TFinanceCustomerAssets)表服务实现类
 *
 * @author Bellamy
 * @since 2020-12-01 10:41:38
 */
@Service("financeCustomerAssetsService")
public class FinanceCustomerAssetsServiceImpl implements FinanceCustomerAssetsService {
    @Autowired
    private FinanceCustomerAssetsDao assetsDao;

    /**
     * 查询余额是否充足
     *
     * @param req
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public Result findAssets(FinanceCustomerAssetsReq req) throws Exception{
        if (!StringUtils.isNotEmpty(req.getAssetsId().toString())) {
            return Result.of_error("资产id为空，查询失败!");
        }
        if (!StringUtils.isNotEmpty(req.getDataPrice())) {
            return Result.of_error("价格为空，查询失败!");
        }
        FinanceCustomerAssets assets = assetsDao.findById(req.getAssetsId());
        if (assets == null) {
            return Result.of_error("查询资产失败!");
        }
        // 可用金额/冻结金额
        BigDecimal useMoney = assets.getUseMoney();
        BigDecimal frozenMoney = assets.getFrozenMoney();
        // 余额不充足
        if(useMoney.compareTo(new BigDecimal(req.getDataPrice())) == -1){
            return Result.of_error("余额不足!");
        }
        // 余额充足
        if (useMoney.compareTo(new BigDecimal(req.getDataPrice())) > -1) {
            // 冻结金额
            useMoney = useMoney.subtract(new BigDecimal(req.getDataPrice()));
            frozenMoney = frozenMoney.add(new BigDecimal(req.getDataPrice()));
            assets.setUseMoney(useMoney);
            assets.setFrozenMoney(frozenMoney);
            assets.setTotalMoney(useMoney);
            assets.setUptTime(DateUtils.getToday());
            assets.setUptPerson(req.getUserId() + "");
            assets.setDelFlag("N");
            assetsDao.updateMoney(assets);
            return Result.of_success("余额充足!");
        }
        return Result.of_error("查询失败!");
    }

    /**
     * 解冻金额扣钱
     *
     * @param req
     * @return
     */
    @Override
    @Transactional(rollbackFor = Exception.class, propagation = Propagation.REQUIRES_NEW)
    public Result unfreezeMoney(FinanceCustomerAssetsReq req) throws Exception {
        Result result = new Result();
        if (req.getAssetsId() == null) {
            return Result.of_error("资产id为空，扣钱失败!");
        }
        if (req.getDataPrice() == null || StringUtils.isBlank(req.getDataPrice())) {
            return Result.of_error("价格为空，扣钱失败!");
        }
        FinanceCustomerAssets assets = assetsDao.findById(req.getAssetsId());
        if (assets == null) {
            return Result.of_error("查询资产失败!");
        }
        BigDecimal price = new BigDecimal(req.getDataPrice());
        if (price.compareTo(BigDecimal.ZERO) < -1) {
            return Result.of_error("扣款金额不能为零或负!");
        }
        BigDecimal useMoney = assets.getUseMoney();
        BigDecimal frozenMoney = assets.getFrozenMoney();
        if (req.getCallStatus() && useMoney.compareTo(BigDecimal.ZERO) == 1  && price.compareTo(new BigDecimal(req.getDataPrice())) == 1) {
            // 扣钱
            useMoney = useMoney.subtract(new BigDecimal(req.getDataPrice()));
            result.setMessage("扣款成功!");
            result.setCode(StatusCode.OK);
        }else if(frozenMoney.compareTo(BigDecimal.ZERO) == 1){
            // 解冻金额
            frozenMoney = frozenMoney.subtract(new BigDecimal(frozenMoney.toString()));
            useMoney = useMoney.add(new BigDecimal(assets.getFrozenMoney().toString()));
            result.setMessage("解冻成功");
            result.setCode(StatusCode.OK);
        }else {
            result.setMessage("金额不足，请充值!");
            result.setCode(StatusCode.ERROR);
        }
        // 更新金额
        assets.setFrozenMoney(frozenMoney);
        assets.setUseMoney(useMoney);
        assets.setTotalMoney(useMoney);
        assets.setUptPerson(req.getUserId() + "");
        assets.setUptTime(DateUtils.getToday());
        assets.setDelFlag("N");
        assetsDao.updateMoney(assets);
        return result;

    }
}