package com.jz.dm.mall.moduls.service.impl;


import com.jz.common.bean.MallCustomerApiDto;
import com.jz.common.constant.RedisMessageConstant;
import com.jz.common.constant.ResultMsg;
import com.jz.common.enums.UserTypeEnum;

import com.jz.common.utils.DateUtils;
import com.jz.common.utils.Result;

import com.jz.dm.mall.moduls.controller.customer.bean.CustomerDto;
import com.jz.dm.mall.moduls.controller.customer.bean.req.CustomerRequest;
import com.jz.dm.mall.moduls.entity.MallCustomer;
import com.jz.dm.mall.moduls.entity.MallMenu;
import com.jz.dm.mall.moduls.mapper.MallCustomerDao;
import com.jz.dm.mall.moduls.service.MallCustomerService;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collectors;


/**
 * 商城用户(TMallCustomer)表服务实现类
 *
 * @author Bellamy
 * @since 2020-12-01 10:41:40
 */
@Service("mallCustomerService")
public class MallCustomerServiceImpl implements MallCustomerService {
    @Autowired
    private MallCustomerDao tMallCustomerDao;

    @Autowired
    private RedisTemplate redisTemplate;

    /**
     * 通过用户账号进行查询
     *
     * @param username
     * @return
     */
    @Override
    public CustomerDto selectByAccount(String username, HttpServletRequest request) {
        CustomerDto mallCustomer = tMallCustomerDao.selectByAccount(username);

//        String customer = JSON.toJSONString(mallCustomer);
        if (mallCustomer != null) {
            MallCustomerApiDto mallCustomerApiDto = new MallCustomerApiDto();
            // 赋值
            mallCustomerApiDto.setCustomerId(mallCustomer.getCustomerId());
            mallCustomerApiDto.setDepartmentId(mallCustomer.getDepartmentId());
            mallCustomerApiDto.setUserTypeEnum(UserTypeEnum.COMPANY_ROLE);
            mallCustomerApiDto.setCustomerAccount(mallCustomer.getCustomerAccount());
            mallCustomerApiDto.setCustomerName(mallCustomer.getCustomerName());
            mallCustomerApiDto.setAssetsId(mallCustomer.getAssetsId());
            // 存入到session
            request.getSession().setAttribute("mallCustomer", mallCustomerApiDto);

        // 存入到redis

            redisTemplate.opsForValue().set("USER_" + RedisMessageConstant.SENDTYPE_LOGIN_CUSTOMER, mallCustomerApiDto, 3, TimeUnit.DAYS);
        }
        return mallCustomer;
    }

    /**
     * 通过手机号进行查询
     *
     * @param username
     * @return
     */
    @Override
    public CustomerDto selectByPhone(String username, HttpServletRequest request) {
        CustomerDto mallCustomer = tMallCustomerDao.selectByPhone(username);
        if (mallCustomer != null) {
            MallCustomerApiDto mallCustomerApiDto = new MallCustomerApiDto();
            // 赋值
            mallCustomerApiDto.setCustomerId(mallCustomer.getCustomerId());
            mallCustomerApiDto.setDepartmentId(mallCustomer.getDepartmentId());
            mallCustomerApiDto.setUserTypeEnum(UserTypeEnum.COMPANY_ROLE);
            mallCustomerApiDto.setCustomerAccount(mallCustomer.getCustomerAccount());
            mallCustomerApiDto.setCustomerName(mallCustomer.getCustomerName());
            mallCustomerApiDto.setAssetsId(mallCustomer.getAssetsId());
            // 存入到session
            request.getSession().setAttribute("mallCustomer", mallCustomerApiDto);

            // 存入到redis
            redisTemplate.opsForValue().set("USER_" + RedisMessageConstant.SENDTYPE_LOGIN_CUSTOMER, mallCustomerApiDto, 3, TimeUnit.DAYS);
        }
        return mallCustomer;
    }

    /**
     * 注册账号
     *
     * @param
     */
    @Override
    public void saveCustomer(CustomerRequest customerRequest) {
        MallCustomer mallCustomer = new MallCustomer();
        // 获取验证码
        String vailCode = customerRequest.getVailCode();
        String username = customerRequest.getUsername();
        String telephone = customerRequest.getTelephone();

        // 从redis获取验证码
        String key = RedisMessageConstant.SENDTYPE_LOGIN + "_" + telephone;
//        String key = "18179617425";
        String codeInRedis = (String) redisTemplate.opsForValue().get(key);
//        String codeInRedis = "147826";
        if (codeInRedis.equals(vailCode)) {
            mallCustomer.setCustomerAccount(username);
            mallCustomer.setPassword(customerRequest.getPassword());
            mallCustomer.setCustomerPhone(telephone);
            mallCustomer.setCreTime(DateUtils.getToday());
            mallCustomer.setRegisterTime(DateUtils.getToday());
            tMallCustomerDao.saveCustomer(mallCustomer);
        }
    }

    /**
     * 通过手机号查询是否已注册
     *
     * @param telephone@return
     */
    @Override
    public MallCustomer selectByPhone(String telephone) {
        MallCustomer mallCustomer = tMallCustomerDao.selectByPhoneExist(telephone);
        if (mallCustomer != null) {
            return mallCustomer;
        }
        return null;
    }

    /**
     * 修改密码
     *
     * @param customerId
     * @param password
     */
    @Override
    public void updatePassword(Long customerId, String password) {
        MallCustomer mallCustomer = tMallCustomerDao.findById(customerId);
        if (mallCustomer != null) {
            MallCustomer customer = new MallCustomer();
            mallCustomer.setPassword(password);
            mallCustomer.setUptTime(DateUtils.getToday());
            mallCustomer.setUptPerson(mallCustomer.getCustomerId() + "");
            mallCustomer.setCustomerId(mallCustomer.getCustomerId());
            tMallCustomerDao.updatePassword(customer);
        }
    }

    /**
     * 通过id查找用户信息
     *
     * @param customerId
     * @return
     */
    @Override
    public MallCustomer selectByUser(Long customerId) {
        MallCustomer mallCustomer = tMallCustomerDao.findById(customerId);
        return mallCustomer;
    }

    /**
     * 根据用户id，姐角色id查找菜单
     *
     * @param userId
     * @param roleId
     * @return
     */
    @Override
    public Result getMenu(Long userId, Long roleId) {

        //查询用户拥有的角色
        List<MallMenu> entities = tMallCustomerDao.getMenu(userId, roleId);

        // 找到所有的一级分类
        List<MallMenu> level1Menus = entities.stream().filter(sysMenuEntity ->
                sysMenuEntity.getParentId() == 0 && sysMenuEntity.getIfChild().equals("N")
        ).map((menu)->{
            menu.setChildren(getChildrens(menu,entities));
            return menu;
        }).collect(Collectors.toList());
        return Result.of_success(ResultMsg.SUCCESS, level1Menus);
    }

    private List<MallMenu> getChildrens(MallMenu sysMenu,List<MallMenu> all) {

        List<MallMenu> children = all.stream().filter(sysMenuEntity -> {
            return sysMenuEntity.getParentId() == sysMenu.getMenuId() && sysMenuEntity.getIfChild().equals("Y");
        }).map(sysMenuEntity -> {
            //1、找到子菜单,同时递归调用
            sysMenuEntity.setChildren(getChildrens(sysMenuEntity, all));
            return sysMenuEntity;
        }).collect(Collectors.toList());

        return children;
    }


}